﻿/*
 * CutPurse by Kamilche
 * 
 * This is an add-on that will:
 * 
 *   1. Open containers containing items, such as clams.
 *   2. Combine scraps of borean leather, motes of air, etc.
 *
 * If you're a rogue, it will also:
 * 
 *   3. Open junkboxes/lockboxes and extract their contents.
 *   4. Poison both your weapons as necessary.
 *   
 * If you're a rogue, get the glyph that makes 'Pick Lock' take
 * no casting time. It goes much faster that way, and that's what
 * this code was tested with.
 *   
 * 2010/11/12  v1.0.0.0 - First version.
 * 2012/08/22  v1.0.0.1 - Second version
 * 2012/09/18  v1.0.0.2 - Moved knife poisoning to separate class.
 * 
 */

using Styx;
using Styx.Common;
using Styx.CommonBot;
using Styx.Helpers;
using Styx.WoWInternals;
using Styx.WoWInternals.WoWObjects;
using Styx.Plugins;

using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Net;
using System.Threading;
using System.Windows.Forms;

namespace CutPurse
{

    public class CutPurse : HBPlugin
    {
        public override string Name { get { return "CutPurse"; } }
        public override string Author { get { return "Kamilche"; } }
        public override Version Version { get { return _version; } }
        private readonly Version _version = new Version(1, 0, 0, 2);


        private static uint[] _lockboxes = new uint[] {
            68729, // Elementium Lockbox
            16882,  // Battered Junkbox
            63349,  // Flame-Scarred Junkbox
            16885,  // Heavy Junkbox
            8249,   // Junkboxes Needed
            43575,  // Reinforced Junkbox
            29569,  // Strong Junkbox
            16884,  // Sturdy Junkbox
            16883,  // Worn Junkbox
            30454,  // Ar'tor's Lockbox
            30646,  // Borak's Lockbox
            5760,   // Eternium Lockbox
            43622,  // Froststeel Lockbox
            7870,   // Gallywix's Lockbox
            30429,  // Grom'tor's Lockbox
            4633,   // Heavy Bronze Lockbox
            4634,   // Iron Lockbox
            31952,  // Khorium Lockbox
            121264, // Lucius's Lockbox
            12192,  // Mist Veil's Lockbox
            175166, // Mist Veil's Lockbox
            5758,   // Mithril Lockbox
            19425,  // Mysterious Lockbox
            4632,   // Ornate Bronze Lockbox
            178244, // Practice Lockbox
            4638,   // Reinforced Steel Lockbox
            187875, // Salrand's Lockbox
            12191,  // Silver Dawning's Lockbox
            175165, // Silver Dawning's Lockbox
            175802, // Small Lockbox
            141980, // Spectral Lockbox
            4637,   // Steel Lockbox
            177790, // Strange Lockbox
            4636,   // Strong Iron Lockbox
            42288,  // Summon Salvaged Lockbox
            5759,   // Thorium Lockbox
            45986,  // Tiny Titanium Lockbox
            43624,  // Titanium Lockbox
            2718,   // Trelane's Lockbox
        };

        private static uint[,] _data = new uint[,] { 
            {1, 5523},   // Small Barnacled Clam
            {1, 5524},   // Thick-shelled Clam
            {1, 7973},   // Big-mouth Clam
            {1, 24476},  // Jaggal Clam
            {1, 51999},  // Level 1 Satchel of Helpful Goods
            {1, 52000},  // Level 2 Satchel of Helpful Goods
            {1, 67248},  // Level 3 Satchel of Helpful Goods
            {1, 52001},  // Level 4 Satchel of Helpful Goods
            {1, 52002},  // Level 5 Satchel of Helpful Goods
            {1, 52003},  // Level 6 Satchel of Helpful Goods
            {1, 52004},  // Level 7 Satchel of Helpful Goods
            {1, 52005},  // Level 8 Satchel of Helpful Goods
            {1, 67250},  // Level 9 Satchel of Helpful Goods
            {1, 67495}, // Strange Bloated Stomach
            {5, 33567},  // Borean Leather Scraps
            {1, 36781},  // Darkwater Clam
            {1, 44700},  // Brooding Darkwater Clam
            {1, 45909},  // Giant Darkwater Clam
            {10, 37700}, // Crystallized Air
            {10, 37701}, // Crystallized Earth
            {10, 37702}, // Crystallized Fire
            {10, 37703}, // Crystallized Shadow
            {10, 37704}, // Crystallized Life
            {10, 37705}, // Crystallized Water
            {10, 22572}, // Mote of Air
            {10, 22573}, // Mote of Earth 
            {10, 22574}, // Mote of Fire
            {10, 22575}, // Mote of Life
            {10, 22576}, // Mote of Mana
            {10, 22577}, // Mote of Shadow
            {10, 22578}, // Mote of Water
            {1, 3352},   // Ooze-covered Bag
            {1, 20768},  // Oozing Bag
            {1, 20767},  // Scum Covered Bag
            {1, 44663},  // Abandoned Adventurer's Satchel
            {1, 27511},  // Inscribed Scrollcase
            {1, 20708},  // Tightly Sealed Trunk
            {1, 6351},   // Dented Crate
            {1, 6352},   // Waterlogged Crate
            {1, 6353},   // Small Chest
            {1, 6356},   // Battered Chest
            {1, 6357},   // Sealed Crate
            {1, 13874},  // Heavy Crate
            {1, 21113},  // Watertight Trunk
            {1, 21150},  // Iron Bound Trunk
            {1, 21228},  // Mithril Bound Trunk
            {1, 27513},  // Curious Crate
            {1, 27481},  // Heavy Supply Crate
            {1, 44475},  // Reinforced Crate
            {1, 52340}, // Abyssal Clam
	        {1, 61387}, // Hidden Stash
	        {1, 67539}, // Tiny Treasure Chest
        };
        private static DateTime NextTime = DateTime.Now;

        private void log(String fmt, params object[] args)
        {
            String s = String.Format(fmt, args);
            log(Color.CornflowerBlue, fmt, args);
        }

        private void log(Color color, String fmt, params object[] args)
        {
            String s = String.Format(fmt, args);
            Logging.Write(System.Windows.Media.Color.FromRgb(color.R, color.G, color.B), String.Format("[{0}]: {1}", Name, s));
            fmt = @"getglobal(""ChatFrame1""):AddMessage(""{0}"", {1}, {2}, {3}, 0);";
            Lua.DoString(string.Format(fmt, s, color.R / 255.0, color.G / 255.0, color.B / 255.0));
            fmt = @"getglobal(""ChatFrame3""):AddMessage(""{0}"", {1}, {2}, {3}, 0);";
            Lua.DoString(string.Format(fmt, s, color.R / 255.0, color.G / 255.0, color.B / 255.0));
        }

        public override void  Initialize()
        {
            KeepAurasUp.Initialize();
     	    base.Initialize();
        }

        public override void Pulse()
        {

            if (DateTime.Now < NextTime ||
                StyxWoW.Me.Mounted ||
                StyxWoW.Me.Combat ||
                StyxWoW.Me.IsDead)
                return;

            NextTime = DateTime.Now.AddSeconds(5);

            KeepAurasUp.Pulse();
            CheckInventoryItems();
        }

        private void CheckInventoryItems()
        {
            // Check all items in inventory
            foreach (WoWItem item in StyxWoW.Me.CarriedItems)
            {
                // Check to see if it can be lockpicked
                if (SpellManager.HasSpell("Pick Lock") && _lockboxes.Contains(item.Entry))
                {
                    SpellManager.Cast("Pick Lock");
                    StyxWoW.SleepForLagDuration();
                    String s = String.Format("UseItemByName(\"{0}\")", item.Name);
                    log("Unlocking {0} guid {1}", item.Name, item.Guid);
                    Lua.DoString(s);
                    StyxWoW.SleepForLagDuration();
                    StyxWoW.SleepForLagDuration();
                    while (StyxWoW.Me.IsCasting)
                        Thread.Sleep(50);
                    log("Opening {0} guid {1}", item.Name, item.Guid);
                    Lua.DoString(s);
                    StyxWoW.SleepForLagDuration();
                }
                else
                {
                    // Check to see if it can be opened or used or combined.
                    for (int i = 0; i <= _data.GetUpperBound(0); i++)
                    {
                        if (_data[i, 1] == item.Entry)
                        {
                            int cnt = Convert.ToInt32(Lua.GetReturnValues(String.Format("return GetItemCount(\"{0}\")", item.Name), Name + ".lua")[0]);
                            int max = (int)(cnt / _data[i, 0]);
                            for (int j = 0; j < max; j++)
                            {
                                String s = String.Format("UseItemByName(\"{0}\")", item.Name);
                                log("Using {0} we have {1}", item.Name, cnt);
                                Lua.DoString(s);
                                StyxWoW.SleepForLagDuration();
                            }
                            break;
                        }
                    }
                }
            }
        }
    }

    public static class KeepAurasUp
    {
        private static DateTime NextTime = DateTime.Now;
        private static Dictionary<String, List<KeyValuePair<String, String>>> values = new Dictionary<String, List<KeyValuePair<String, String>>>();

        public static void Initialize()
        {
            AddAura("Lethal", "Deadly Poison");
            AddAura("Lethal", "Wound Poison");
            AddAura("Nonlethal", "Leeching Poison");
            AddAura("Nonlethal", "Crippling Poison");
            AddAura("Nonlethal", "Mind-Numbing Poison");
            AddAura("Nonlethal", "Paralytic Poison");
            AddAura("MOTW", "Mark of the Wild");
            AddAura("MOTW", "Blessing of Kings");
            AddAura("MOTW", "Legacy of the Emperor");
        }

        public static void Pulse()
        {
            if (DateTime.Now < NextTime)
                return;

            NextTime = DateTime.Now.AddSeconds(5);

            if (StyxWoW.Me.Mounted ||
                StyxWoW.Me.Combat ||
                StyxWoW.Me.IsDead)
                return;

            // Check each buff group in the list
            foreach (KeyValuePair<String, List<KeyValuePair<String, String>>> kvp in values)
            {
                bool hasaura = false;
                String buffname = kvp.Key;
                // Check each buff in the buff group
                foreach (KeyValuePair<String, String> buff in kvp.Value)
                {
                    if (StyxWoW.Me.HasAura(buff.Value))
                    {
                        hasaura = true;
                        break;
                    }
                }
                if (hasaura == false)
                {
                    // Cast the first spell you have in the list
                    foreach (KeyValuePair<String, String> buff in kvp.Value)
                    {
                        String spellname = buff.Key;
                        String auraname = buff.Value;
                        if (SpellManager.Spells.ContainsKey(spellname))
                        {
                            WoWMovement.MoveStop();
                            if (spellname == auraname)
                                log("[KeepAurasUp] {0}: Casting {1}", buffname, spellname);
                            else
                                log("[KeepAurasUp] {0}: Casting {1} to get {2}", buffname, spellname, auraname);
                            SpellManager.Spells[spellname].Cast();
                            while (StyxWoW.Me.IsCasting)
                                StyxWoW.SleepForLagDuration();
                            break;
                        }
                    }
                }
            }
        }

        private static void AddAura(String Name, String spellname)
        {
            AddAura(Name, spellname, spellname);
        }

        private static void AddAura(String Name, String spellname, String auraname)
        {
            List<KeyValuePair<String, String>> lst = null;

            if (values.ContainsKey(Name))
                lst = values[Name];

            if (lst == null)
            {
                lst = new List<KeyValuePair<String, String>>();
                values.Add(Name, lst);
            }

            lst.Add(new KeyValuePair<String, String>(spellname, auraname));
        }

        private static void log(String fmt, params object[] args)
        {
            String s = String.Format(fmt, args);
            Color color = Color.Aquamarine;
            Logging.Write(System.Windows.Media.Color.FromRgb(color.R, color.G, color.B), String.Format("{0}", s));
            fmt = @"getglobal(""ChatFrame1""):AddMessage(""{0}"", {1}, {2}, {3}, 0);";
            Lua.DoString(string.Format(fmt, s, color.R / 255.0, color.G / 255.0, color.B / 255.0));
            fmt = @"getglobal(""ChatFrame3""):AddMessage(""{0}"", {1}, {2}, {3}, 0);";
            Lua.DoString(string.Format(fmt, s, color.R / 255.0, color.G / 255.0, color.B / 255.0));
        }
    }
}



